from openai import AsyncOpenAI

from ragbits.core.prompt import Prompt
from ragbits.guardrails.base import Guardrail, GuardrailVerificationResult


class OpenAIModerationGuardrail(Guardrail):
    """
    Guardrail based on OpenAI moderation
    """

    def __init__(self, moderation_model: str = "omni-moderation-latest"):
        self._openai_client = AsyncOpenAI()
        self._moderation_model = moderation_model

    async def verify(self, input_to_verify: Prompt | str) -> GuardrailVerificationResult:
        """
        Verifies whether provided input meets certain criteria

        Args:
            input_to_verify: prompt or output of the model to check

        Returns:
            verification result
        """
        if isinstance(input_to_verify, Prompt):
            inputs = [{"type": "text", "text": input_to_verify.rendered_user_prompt}]

            if input_to_verify.rendered_system_prompt is not None:
                inputs.append({"type": "text", "text": input_to_verify.rendered_system_prompt})

            if attachments := input_to_verify.attachments:
                messages = [input_to_verify.create_message_with_attachment(attachment) for attachment in attachments]
                inputs.extend(messages)
        else:
            inputs = [{"type": "text", "text": input_to_verify}]
        response = await self._openai_client.moderations.create(model=self._moderation_model, input=inputs)  # type: ignore

        fail_reasons = [result for result in response.results if result.flagged]
        return GuardrailVerificationResult(
            guardrail_name=self.__class__.__name__,
            succeeded=len(fail_reasons) == 0,
            fail_reason=None if len(fail_reasons) == 0 else str(fail_reasons),
        )



import os
from unittest.mock import AsyncMock, patch

from pydantic import BaseModel

from ragbits.guardrails.base import GuardrailManager, GuardrailVerificationResult
from ragbits.guardrails.openai_moderation import OpenAIModerationGuardrail


class MockedModeration(BaseModel):
    flagged: bool
    fail_reason: str | None


class MockedModerationCreateResponse(BaseModel):
    results: list[MockedModeration]


async def test_manager():
    guardrail_mock = AsyncMock()
    guardrail_mock.verify.return_value = GuardrailVerificationResult(
        guardrail_name=".", succeeded=True, fail_reason=None
    )
    manager = GuardrailManager([guardrail_mock])
    results = await manager.verify("test")
    assert guardrail_mock.verify.call_count == 1
    assert len(results) == 1


@patch.dict(os.environ, {"OPENAI_API_KEY": "."}, clear=True)
async def test_not_flagged():
    guardrail = OpenAIModerationGuardrail()
    guardrail._openai_client = AsyncMock()
    guardrail._openai_client.moderations.create.return_value = MockedModerationCreateResponse(
        results=[MockedModeration(flagged=False, fail_reason=None)]
    )
    results = await guardrail.verify("Test")
    assert results.succeeded is True
    assert results.fail_reason is None
    assert results.guardrail_name == "OpenAIModerationGuardrail"


@patch.dict(os.environ, {"OPENAI_API_KEY": "."}, clear=True)
async def test_flagged():
    guardrail = OpenAIModerationGuardrail()
    guardrail._openai_client = AsyncMock()
    guardrail._openai_client.moderations.create.return_value = MockedModerationCreateResponse(
        results=[MockedModeration(flagged=True, fail_reason="Harmful content")]
    )
    results = await guardrail.verify("Test")
    assert results.succeeded is False
    assert results.fail_reason == "[MockedModeration(flagged=True, fail_reason='Harmful content')]"
    assert results.guardrail_name == "OpenAIModerationGuardrail"
